/**
 * Copyright (c) 2000 GEMPLUS group. All Rights Reserved.
 *------------------------------------------------------------------------------
 *  Project name    :  GemXpresso RAD III  Environment
 *                      - Java Card applet -
 *
 *  Platform        :  Java virtual machine
 *  Language        :  1.3.0-C
 *  Devl tool       :  Borland (c) JBuilder 5.0 and 6.0
 *
 *  Original author : Adalbert Ochotta, Florian Muecke
 *  Date            : Wed Apr 27 14:56:23 CEST 2005
 *------------------------------------------------------------------------------
 */

/*
 * Package name
 */
package MasterMind;

/*
 * Imported packages
 */
// specific import for Javacard API access
import javacard.framework.*;
import javacard.security.RandomData;

public class MMApplet extends javacard.framework.Applet
{
    public static final byte GUESS_OFFSET = ISO7816.OFFSET_CDATA;
    public static final byte STATUS_OFFSET = ISO7816.OFFSET_CDATA + 4;
    public static final byte INS_CHECK      = (byte)0x02;
    private RandomData rnd = RandomData.getInstance(RandomData.ALG_SECURE_RANDOM);
    private byte[] master;  // 0=rot, 1=grn, 2=gelb, 3=blau, 4=orange, 5=violett
    private short tries;


    /**
     * MMApplet default constructor
     * Only this class's install method should create the applet object.
     */
    protected MMApplet(byte[] buffer, short offset, byte length)
    {
        // data offset is used for application specific parameter.
        // initialization with default offset (AID offset).
        short dataOffset = offset;
        boolean isOP2 = false;
        master = new byte[4];

        if(length > 9) {
            // Install parameter detail. Compliant with OP 2.0.1.

            // | size | content
            // |------|---------------------------
            // |  1   | [AID_Length]
            // | 5-16 | [AID_Bytes]
            // |  1   | [Privilege_Length]
            // | 1-n  | [Privilege_Bytes] (normally 1Byte)
            // |  1   | [Application_Proprietary_Length]
            // | 0-m  | [Application_Proprietary_Bytes]

            // shift to privilege offset
            dataOffset += (short)( 1 + buffer[offset]);
            // finally shift to Application specific offset
            dataOffset += (short)( 1 + buffer[dataOffset]);

           // <IF NECESSARY, USE COMMENTS TO CHECK LENGTH >
           // // checks wrong data length
           // if(buffer[dataOffset] !=  <PUT YOUR PARAMETERS LENGTH> )
           //     // return received proprietary data length in the reason
           //     ISOException.throwIt((short)(ISO7816.SW_WRONG_LENGTH + offset + length - dataOffset));

            // go to proprietary data
            dataOffset++;

            // update flag
            isOP2 = true;

        } else {
           // <IF NECESSARY, USE COMMENTS TO CHECK LENGTH >
           // if(length != <PUT YOUR PARAMETERS LENGTH> )
           //     ISOException.throwIt((short)(ISO7816.SW_WRONG_LENGTH + length));
       }

        // <PUT YOUR CREATION ACTION HERE>

        // register this instance
        if (isOP2)
          register(buffer, (short)(offset + 1), (byte)buffer[offset]);
        else
          register();
    }

    /**
     * Method installing the applet.
     * @param bArray the array constaining installation parameters
     * @param bOffset the starting offset in bArray
     * @param bLength the length in bytes of the data parameter in bArray
     */
    public static void install(byte[] bArray, short bOffset, byte bLength) throws ISOException
    {
        /* applet  instance creation */
        MMApplet myApplet = new MMApplet (bArray, bOffset, (byte)bLength );
        myApplet.newGame();

    }

    /**
     * Select method returns true if applet selection is supported.
     * @return boolean status of selection.
     */
    public boolean select()
    {
        // <PUT YOUR SELECTION ACTION HERE>

        // return status of selection
        return true;
    }

    /**
     * Deselect method called by the system in the deselection process.
     */
    public void deselect()
    {

        // <PUT YOUR DESELECTION ACTION HERE>

        return;
    }

    /**
     * Method processing an incoming APDU.
     * @see APDU
     * @param apdu the incoming APDU
     * @exception ISOException with the response bytes defined by ISO 7816-4
     */
    public void process(APDU apdu) throws ISOException
    {
        // get the APDU buffer
        byte[] apduBuffer = apdu.getBuffer();

        // ignore the applet select command dispached to the process
        if (selectingApplet())
            return;

        // APDU instruction parser
        switch ( apduBuffer[ISO7816.OFFSET_INS] )
        {
            case INS_CHECK: check(apdu); return;

            default :
                // The INS code is not supported by the dispatcher
                ISOException.throwIt( ISO7816.SW_INS_NOT_SUPPORTED ) ;
            break ;

        }
    }

     // testet guess, liefert true, wenn guess richtig
    public void check(APDU apdu) throws ISOException {
        short len = apdu.setIncomingAndReceive();
        if (len != 4) ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        byte[] buffer = apdu.getBuffer();
        byte blacks = 0;
        byte whites = 0;

        for (byte i=0; i <= 3; i++){
            if (master[i] == buffer[(short) GUESS_OFFSET + i]) {
                blacks++;
                buffer[(short) STATUS_OFFSET + i] = 1;
                buffer[(short) GUESS_OFFSET + i] = -1;
            }
        }
        if (blacks == 4) {
            Util.setShort(buffer, (short)(ISO7816.OFFSET_CDATA + 2), ++tries);
            newGame();
        }
        else {
          for (byte i=0; i <= 3; i++){
              if  (buffer[(short) STATUS_OFFSET + i] == 0) {
                  for (byte j=0; j <= 3; j++) {
                      if (i == j) continue;
                      if (master[i] == buffer[(short) GUESS_OFFSET + j]) {
                          whites++;
                          buffer[(short) GUESS_OFFSET + j] = -1;
                          break;
                      }
                  }
              }
          }
          Util.setShort(buffer, (short)(ISO7816.OFFSET_CDATA + 2), ++tries);
        }
        buffer[(short) ISO7816.OFFSET_CDATA] = blacks;
        buffer[(short) ISO7816.OFFSET_CDATA + 1] = whites;
        apdu.setOutgoing();
        apdu.setOutgoingLength((short) 4);
        apdu.sendBytes(ISO7816.OFFSET_CDATA, (short) 4);
    }

     private void newGame() {
        tries = 0;
        rnd.generateData(master, (short) 0, (short) 4);
        for (byte i = 0; i <= 3; i++) {
          master[i] = (byte)((master[i] & 127) % 6);
        }
    }
}