package Mastermind;

import java.awt.*;
import java.awt.event.*;

class UI extends Frame implements MouseListener, MouseMotionListener {
    Engine engine;

    boolean init = true;
    Image buf;
    Graphics bufg;
    Point mPoint;
    int i, j, k;
    int curRow, tries;
    byte dots[], points[];

    Color lights[], darks[], darkGray;

    public UI() {
        super("Mastermind");
        engine = new Engine();

        dots = new byte[40];
        points = new byte[40];

        lights = new Color[6];
        darks = new Color[6];

        darkGray = new Color(60, 60, 60);
        darks[0] = new Color(0, 0, 255); // Blue
        lights[0] = new Color(128, 128, 255);
        darks[1] = new Color(0, 192, 0); // Green
        lights[1] = new Color(64, 255, 64);
        darks[2] = new Color(255, 0, 0); // Red
        lights[2] = new Color(255, 128, 128);
        darks[3] = new Color(255, 192, 0); // Yellow
        lights[3] = new Color(255, 255, 70);
        darks[4] = new Color(210, 150, 0); // Orange
        lights[4] = new Color(245, 185, 0);
        darks[5] = new Color(255, 0, 255); // Purple
        lights[5] = new Color(255, 128, 255);

        setUndecorated(true);
        setSize(126, 294);
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screen.width - getBounds().width) / 2, (screen.height - getBounds().height) / 2);
        setResizable(false);
        setVisible(true);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                e.getWindow().dispose();
            }
        });
        addMouseListener(this);
        addMouseMotionListener(this);
        buf = createImage(126, 294);
        bufg = buf.getGraphics();
        init = false;
        newGame();
    }

    public void mouseMoved(MouseEvent e) {}

    public void mouseDragged(MouseEvent e) {
        int x, y;
        if (mPoint == null)return;
        x = getLocation().x + (e.getPoint().x - mPoint.x);
        y = getLocation().y + (e.getPoint().y - mPoint.y);
        setLocation(x, y);
    }

    public void mousePressed(MouseEvent e) {
        mPoint = e.getPoint();
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
        int x = e.getPoint().x;
        int y = e.getPoint().y;

        if (e.getClickCount() == 2) {
            //dispose();
        }
        if (x > 51 && x < 94 && y > 279) {
            dispose();
        }
        if (x < 50 && y > 279) {
            check();
        }

        if (x > 6 && x < 92) {
            if ((y < (274 - curRow * 27)) && (y > (249 - curRow * 27))) {
                int curCol = -1;
                if (x < 27) curCol = 0;
                if (x > 27 && x < 48) curCol = 1;
                if (x > 48 && x < 70) curCol = 2;
                if (x > 70) curCol = 3;
                if (curCol == -1) return;
                byte curDot = dots[curRow * 4 + curCol];
                if (e.getButton() == e.BUTTON1) curDot++;
                if (e.getButton() == e.BUTTON3) curDot--;
                if (curDot == 6) curDot = 0;
                if (curDot <= -1) curDot = 5;
                dots[curRow * 4 + curCol] = curDot;
                repaint();
            }

        }
    }


    public void paint(Graphics g) {
        if (init)return;

        bufg.setColor(Color.gray);
        bufg.fillRect(0, 0, 126, 300);
        bufg.fill3DRect(0, 0, 126, 279, true);
        for (i = 0; i < 10; i++) {
            bufg.draw3DRect(5, 5 + i * 27, 88, 25, false);
            bufg.draw3DRect(95, 5 + i * 27, 25, 25, false);
        }
        bufg.fill3DRect(0, 279, 50, 15, true);
        bufg.fill3DRect(50, 279, 45, 15, true);


        for (i = 0; i < 10; i++) {
            for (j = 0; j < 4; j++) {
                if (dots[i * 4 + j] < 0) {
                    boardHole(bufg, 8 + j * 21, 252 - i * 27);
                } else {
                    boardDot(bufg, 8 + j * 21, 252 - i * 27, dots[i * 4 + j]);
                }
            }
            for (j = 0; j < 2; j++) {
                for (k = 0; k < 2; k++) {
                    if (points[i * 4 + j * 2 + k] < 0) {
                        pointHole(bufg, 96 + k * 12, 249 - i * 27 + j * 12);
                    } else {
                        pointDot(bufg, 96 + k * 12, 249 - i * 27 + j * 12,
                                                   points[i * 4 + j * 2 + k]);
                    }
                }
            }
        }
        bufg.setColor(Color.black);
        bufg.setFont(new Font("Arial", Font.BOLD, 11));
        bufg.drawString("Exit", 64, 290);
        bufg.drawString("Guess!", 6, 290);
        bufg.setColor(Color.red);
        bufg.drawString(String.valueOf(tries), 100, 290);
        g.drawImage(buf, 0, 0, this);
    }

public void check() {
    for (i = 0; i < 4; i++) {
        if (dots[curRow * 4 + i] == -1)return;
    }
    byte[] guess = {dots[curRow * 4], dots[curRow * 4 + 1],
                   dots[curRow * 4 + 2], dots[curRow * 4 + 3]};
    byte[] retVal = engine.check(guess);
    int c = 0;
    for (i = 0; i < retVal[0]; i++) {
        points[curRow * 4 + c] = 1;
        c++;
    }
    if (c == 4) {
        newGame();
        return;
    }
    for (i = 0; i < retVal[1]; i++) {
        points[curRow * 4 + c] = 0;
        c++;
    }

    tries = retVal[2];
    curRow++;
    if (curRow == 10) {
        for(i = 0; i < 36; i++) {
            dots[i] = dots[i + 4];
            points[i] = points[i + 4];
        }
        for(i = 36; i < 40; i++) {
            dots[i] = -1;
            points[i] = -1;
        }

        curRow = 9;
    }
    repaint();
}

    public void newGame() {
        for (i = 0; i < 40; i++) {
            dots[i] = -1;
            points[i] = -1;
        }
        curRow = 0;
        tries = 0;
        repaint();
    }

    public void fixBox(Graphics g, int x, int y, int w, int h, Color c) {
        g.setColor(c);
        g.fillRect(x, y, w, h);
    }

    public void fixDisc(Graphics g, int x, int y, int r, Color c) {
        g.setColor(c);
        g.fillOval(x, y, r, r);
    }

    public void fixCircle(Graphics g, int x, int y, int r, Color c) {
        g.setColor(c);
        g.drawOval(x, y, r, r);
    }

    public void boardDot(Graphics g, int x, int y, int c) {
        fixDisc(g, x + 2, y + 2, 18, darkGray);
        fixDisc(g, x, y, 18, darks[c]);
        fixDisc(g, x + 1, y + 1, 14, lights[c]);
        fixDisc(g, x + 3, y + 3, 6, Color.white);
    }

    public void boardHole(Graphics g, int x, int y) {
        fixDisc(g, x + 5, y + 5, 7, Color.black);
        g.setColor(Color.lightGray);
        g.drawArc(x + 4, y + 4, 9, 9, -135, 180);
    }

    public void pointDot(Graphics g, int x, int y, int c) {
        fixDisc(g, x + 2, y + 2, 10, Color.black);
        if (c == 1) {
            fixDisc(g, x, y, 10, darkGray);
            g.setColor(Color.white);
            g.drawArc(x + 2, y + 2, 7, 7, 70, 130);
        } else {
            fixDisc(g, x, y, 10, Color.white);
            fixDisc(g, x + 3, y + 3, 4, Color.lightGray);
            fixDisc(g, x + 5, y + 5, 4, Color.white);
        }
    }

    public void pointHole(Graphics g, int x, int y) {
        fixDisc(g, x + 3, y + 3, 4, Color.black);
        g.setColor(Color.lightGray);
        g.drawArc(x + 2, y + 2, 6, 6, -135, 180);
    }

    public static void main(String args[]) {
        UI ui = new UI();
    }
}
