/**
 * Copyright (c) 2000 GEMPLUS group. All Rights Reserved.
 *------------------------------------------------------------------------------
 *  Project name    :  GemXpresso RAD III  Environment
 *                      - Java Card applet -
 *
 *  Platform        :  Java virtual machine
 *  Language        :  1.3.0-C
 *  Devl tool       :  Borland (c) JBuilder 5.0 and 6.0
 *
 *  Original author : Florian Mcke, Adalbert Ochotta
 *  Date            : Fri May 20 23:59:50 CEST 2005
 *------------------------------------------------------------------------------
 */

/*
 * Package name
 */
package CopyCard;

/*
 * Imported packages
 */
// specific import for Javacard API access
import javacard.framework.*;
import javacard.security.RandomData;
import javacard.security.*;
import javacardx.crypto.*;

public class CCApplet extends javacard.framework.Applet {

    private static final byte CLA_COPYCARD = (byte) 0x90;

    private static final byte INS_BALANCE = (byte) 0x02;
    private static final byte INS_LOAD = (byte) 0x04;
    private static final byte INS_PAY = (byte) 0x06;

    private static final byte INS_AUTH_CARD = (byte) 0x10;
    private static final byte INS_AUTH_TERM = (byte) 0x12;

    /**
     * Das Credit-Limit das nicht berschritten werden darf. 50.000
     */
    private static final short CREDIT_LIMIT = (short) 0xC350;

    /**
     * Exception wenn zu viel geladen werden soll.
     */
    private static final short ERR_OVER_LIMIT = 0x6600;

    /**
     * Exception wenn zu wenig Credit vorhanden ist.
     */
    private static final short ERR_LOW_CREDIT = 0x6601;

    /**
     * Exception wenn sich Terminal nicht authentifizieren konnte.
     */
    private static final short ERR_NO_AUTH = 0x6610;

    private static final short ERR_WRONG_NONCE = 0x6612;


    /**
     * Das aktuelle Guthaben der Karte.
     */
    private short credit = (short) 0;


    /**
     * True wenn Karte und Terminal authentifiziert sind.
     */
    private boolean auth;

    private RandomData rnd = RandomData.getInstance(RandomData.
            ALG_SECURE_RANDOM);
    byte[] nonce = new byte[6];
    byte[] tmp = new byte[6];

    private javacardx.crypto.Cipher cip;
    private DESKey key;

    /**
     * Key fr die DESede-Verschlsselung.
     */
    private final static byte[] desede_key = {(byte) 0x4C, (byte) 0x8F,
                                             (byte) 0xEC, (byte) 0x6E,
                                             (byte) 0x61, (byte) 0x9E,
                                             (byte) 0xC2, (byte) 0xA2,
                                             (byte) 0x8C, (byte) 0xBC,
                                             (byte) 0xE6,
                                             (byte) 0x6B, (byte) 0x62,
                                             (byte) 0x3D, (byte) 0x97,
                                             (byte) 0xDF, (byte) 0x13,
                                             (byte) 0x68,
                                             (byte) 0xA4, (byte) 0x08,
                                             (byte) 0x7C, (byte) 0x13,
                                             (byte) 0xF8, (byte) 0x1F};


    /**
     * CCApplet default constructor
     * Only this class's install method should create the applet object.
     */
    protected CCApplet(byte[] buffer, short offset, byte length) {
        // data offset is used for application specific parameter.
        // initialization with default offset (AID offset).
        short dataOffset = offset;
        boolean isOP2 = false;

        key = (DESKey) KeyBuilder.buildKey(KeyBuilder.TYPE_DES,
                                           KeyBuilder.LENGTH_DES3_3KEY, false);
        key.setKey(desede_key, (short) 0);
        cip = Cipher.getInstance(Cipher.ALG_DES_ECB_NOPAD, false);

        if (length > 9) {
            // Install parameter detail. Compliant with OP 2.0.1.

            // | size | content
            // |------|---------------------------
            // |  1   | [AID_Length]
            // | 5-16 | [AID_Bytes]
            // |  1   | [Privilege_Length]
            // | 1-n  | [Privilege_Bytes] (normally 1Byte)
            // |  1   | [Application_Proprietary_Length]
            // | 0-m  | [Application_Proprietary_Bytes]

            // shift to privilege offset
            dataOffset += (short) (1 + buffer[offset]);
            // finally shift to Application specific offset
            dataOffset += (short) (1 + buffer[dataOffset]);

            // <IF NECESSARY, USE COMMENTS TO CHECK LENGTH >
            // // checks wrong data length
            // if(buffer[dataOffset] !=  <PUT YOUR PARAMETERS LENGTH> )
            //     // return received proprietary data length in the reason
            //     ISOException.throwIt((short)(ISO7816.SW_WRONG_LENGTH + offset + length - dataOffset));

            // go to proprietary data
            dataOffset++;

            // update flag
            isOP2 = true;

        } else {
            // <IF NECESSARY, USE COMMENTS TO CHECK LENGTH >
            // if(length != <PUT YOUR PARAMETERS LENGTH> )
            //     ISOException.throwIt((short)(ISO7816.SW_WRONG_LENGTH + length));
        }

        // <PUT YOUR CREATION ACTION HERE>

        // register this instance
        if (isOP2)
            register(buffer, (short) (offset + 1), (byte) buffer[offset]);
        else
            register();
    }

    /**
     * Method installing the applet.
     * @param bArray the array constaining installation parameters
     * @param bOffset the starting offset in bArray
     * @param bLength the length in bytes of the data parameter in bArray
     */
    public static void install(byte[] bArray, short bOffset, byte bLength) throws
            ISOException {
        /* applet  instance creation */
        new CCApplet(bArray, bOffset, (byte) bLength);
    }

    /**
     * Select method returns true if applet selection is supported.
     * @return boolean status of selection.
     */
    public boolean select() {
        auth = false;

        // return status of selection
        return true;
    }

    /**
     * Deselect method called by the system in the deselection process.
     */
    public void deselect() {
        auth = false;

        return;
    }

    /**
     * Method processing an incoming APDU.
     * @see APDU
     * @param apdu the incoming APDU
     * @exception ISOException with the response bytes defined by ISO 7816-4
     */
    public void process(APDU apdu) throws ISOException {
        // get the APDU buffer
        byte[] apduBuffer = apdu.getBuffer();

        // ignore the applet select command dispached to the process
        if (selectingApplet())return;

        if (apduBuffer[ISO7816.OFFSET_CLA] != CLA_COPYCARD)
            ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);

        // APDU instruction parser
        switch (apduBuffer[ISO7816.OFFSET_INS]) {
        case INS_LOAD:
            if (auth) load(apdu, apduBuffer);
            else ISOException.throwIt(ERR_NO_AUTH);
            return;

        case INS_PAY:
            if (auth) pay(apdu, apduBuffer);
            else ISOException.throwIt(ERR_NO_AUTH);
            return;

        case INS_BALANCE:
            if (auth) balance(apdu, apduBuffer);
            else ISOException.throwIt(ERR_NO_AUTH);
            return;

        case INS_AUTH_CARD:
            authCard(apdu, apduBuffer);
            return;

        case INS_AUTH_TERM:
            authTerm(apdu, apduBuffer);
            return;

        default:
            // The INS code is not supported by the dispatcher
            ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
            break;

        }
    }


    /**
     * Vergleicht 2 shorts binr und liefert true wenn der erste kleinergleich dem zweiten ist.
     */
    private boolean lessEqual(short x, short y) {
        if ((x & (short) 0x8000) == (y & (short) 0x8000))return (x <= y);
        else return ((y & (short) 0x8000) != 0 ? true : false);
    }


    /**
     * Ldt die in der APDU bergebenen Cents auf die Karte.
     * @param apdu APDU
     * @param apduBuffer byte[]
     */
    public void load(APDU apdu, byte[] apduBuffer) {
        short length = apdu.setIncomingAndReceive();
        if (apduBuffer[ISO7816.OFFSET_LC] == length && length == 8) {
            cip.init(key, Cipher.MODE_DECRYPT);

            // entschlsseln
            cip.doFinal(apduBuffer, (short) ISO7816.OFFSET_CDATA, (short) 8,
                        apduBuffer, (short) ISO7816.OFFSET_CDATA);

            if (Util.arrayCompare(apduBuffer, (short) ISO7816.OFFSET_CDATA, nonce,
                                  (short) 0, (short) 6) != 0) {
                auth = false;
                ISOException.throwIt(ERR_WRONG_NONCE);
            }

            short load = Util.getShort(apduBuffer, (byte) (ISO7816.OFFSET_CDATA + 6));
            if (lessEqual(load, (short) (CREDIT_LIMIT - credit))) {
                credit += load;
            } else ISOException.throwIt(ERR_OVER_LIMIT);
        } else ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        balance(apdu, apduBuffer);
    }


    /**
     * Zieht die zur Bezahlung bentigten Cents von der Karte ab.
     * @param apdu APDU
     * @param apduBuffer byte[]
     */
    public void pay(APDU apdu, byte[] apduBuffer) {
        short length = apdu.setIncomingAndReceive();
        if (apduBuffer[ISO7816.OFFSET_LC] == length && length == 8) {
            cip.init(key, Cipher.MODE_DECRYPT);

            // entschlsseln
            cip.doFinal(apduBuffer, (short) ISO7816.OFFSET_CDATA, (short) 8,
                        apduBuffer, (short) ISO7816.OFFSET_CDATA);

            if (Util.arrayCompare(apduBuffer, (short) ISO7816.OFFSET_CDATA, nonce,
                                  (short) 0, (short) 6) != 0) {
                auth = false;
                ISOException.throwIt(ERR_WRONG_NONCE);
            }

            short pay = Util.getShort(apduBuffer, (byte) (ISO7816.OFFSET_CDATA + 6));
            if (lessEqual(pay, credit)) {
                credit -= pay;
            } else ISOException.throwIt(ERR_LOW_CREDIT);
        } else ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        balance(apdu, apduBuffer);
    }


    /**
     * Gibt das Guthaben zurck.
     * @param apdu APDU
     * @param apduBuffer byte[]
     */
    public void balance(APDU apdu, byte[] apduBuffer) {
        Util.setShort(apduBuffer, (short) 0, credit);
        apdu.setOutgoingAndSend((short) 0, (short) 2);
    }

    /**
     * Methode zur Authentifizierung der Karte gegenber dem Terminal.
     * @param apdu APDU
     * @param apduBuffer byte[]
     */
    public void authCard(APDU apdu, byte[] apduBuffer) {
        short length = apdu.setIncomingAndReceive();
        if (apduBuffer[ISO7816.OFFSET_LC] == length && length == 6) {
            Util.arrayCopy(apduBuffer, (short) (ISO7816.OFFSET_CDATA),
                          nonce, (short) 0, (short) 6);

            // Zufallszahl erzeugen (Lnge 6 Bytes) und in den apduBuffer schreiben
            rnd.generateData(tmp, (short) 0, (short) 6);
            Util.arrayCopy(tmp, (short) 0, apduBuffer,
                           (short) (ISO7816.OFFSET_CDATA + 6), (short) 6);

            cip.init(key, Cipher.MODE_ENCRYPT);

            // Zufallszahl verschlsseln
            cip.doFinal(apduBuffer, (short) ISO7816.OFFSET_CDATA, (short) 8,
                        apduBuffer, (short) ISO7816.OFFSET_CDATA);

            //verschlsselte und eigene Zufallszahl ans Terminal schicken
            apdu.setOutgoingAndSend((short) ISO7816.OFFSET_CDATA, (short) 12);
        } else ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
    }

    /**
     * Methode zur Authentifizierung des Terminals gegenber der Karte.
     * @param apdu APDU
     * @param apduBuffer byte[]
     * @return boolean gibt true zurck, falls sich das Terminal authentifiziert hat.
     */
    public void authTerm(APDU apdu, byte[] apduBuffer) {
        short length = apdu.setIncomingAndReceive();
        if (apduBuffer[ISO7816.OFFSET_LC] == length && length == 8) {
            cip.init(key, Cipher.MODE_DECRYPT);

            // Zufallszahl entschlsseln
            cip.doFinal(apduBuffer, (short) ISO7816.OFFSET_CDATA, (short) 8,
                        apduBuffer, (short) ISO7816.OFFSET_CDATA);
            if (Util.arrayCompare(apduBuffer, (short) ISO7816.OFFSET_CDATA, tmp,
                                  (short) 0, (short) 6) == 0) {
                auth = true;
            } else {
                ISOException.throwIt(ERR_NO_AUTH);
            }
        } else ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
    }
}