package CopyCard;

import swt.terminalinterface.*;
import swt.util.*;

import javax.swing.*;
import javax.swing.JOptionPane.*;

import java.security.*;
import javax.crypto.*;
import java.math.BigInteger;


public class Engine {

    private static final byte CLA_COPYCARD = (byte) 0x90;

    private static final byte INS_BALANCE = (byte) 0x02;
    private static final byte INS_LOAD = (byte) 0x04;
    private static final byte INS_PAY = (byte) 0x06;

    private static final byte INS_AUTH_CARD = (byte) 0x10;
    private static final byte INS_AUTH_TERM = (byte) 0x12;

    private static final short ERR_OVER_LIMIT = 0x6600;
    private static final short ERR_LOW_CREDIT = 0x6601;
    private static final short ERR_NO_AUTH = 0x6610;
    private static final short ERR_WRONG_NONCE = 0x6612;


    public SmartCard sc;
    private SCThread scthread;

    private javax.crypto.Cipher cip;
    private javax.crypto.SecretKey the_key;
    private final static String cryptProvider = "BC";

    private byte authFailed = 0;
    private byte[] nonce = new byte[6];

    /**
     * Schlssel fr die DESede-Verschlsselung
     */
    private final static byte[] desede_key = {(byte) 0x4C, (byte) 0x8F,
                                             (byte) 0xEC, (byte) 0x6E,
                                             (byte) 0x61, (byte) 0x9E,
                                             (byte) 0xC2, (byte) 0xA2,
                                             (byte) 0x8C, (byte) 0xBC,
                                             (byte) 0xE6,
                                             (byte) 0x6B, (byte) 0x62,
                                             (byte) 0x3D, (byte) 0x97,
                                             (byte) 0xDF, (byte) 0x13,
                                             (byte) 0x68,
                                             (byte) 0xA4, (byte) 0x08,
                                             (byte) 0x7C, (byte) 0x13,
                                             (byte) 0xF8, (byte) 0x1F};


    public Engine() {
        scthread = new SCThread(this);
        scthread.start();

        try {
            java.security.Security.addProvider(new org.bouncycastle.jce.
                                               provider.BouncyCastleProvider());
            javax.crypto.SecretKeyFactory sessionkf = javax.crypto.
                    SecretKeyFactory.getInstance("DESede", cryptProvider);
            the_key = sessionkf.generateSecret(new javax.crypto.spec.
                                               DESedeKeySpec(desede_key));
            cip = javax.crypto.Cipher.getInstance("DESede/ECB/NoPadding",
                                                  cryptProvider);

        } catch (NoSuchProviderException e) {
            System.out.println(e);
        } catch (NoSuchAlgorithmException e) {
            System.out.println(e);
        } catch (InvalidKeyException e) {
            System.out.println(e);
        } catch (java.security.spec.InvalidKeySpecException e) {
            System.out.println(e);
        } catch (javax.crypto.NoSuchPaddingException e) {
            System.out.println(e);
        }

    }

    public void dispose() {
//      if(scthread.isAlive()) scthread.stop();
    }


/*    public void key() {
        try {
            javax.crypto.KeyGenerator keygen
                    = javax.crypto.KeyGenerator.getInstance("DESede",
                    cryptProvider);
            keygen.init(192);
            javax.crypto.SecretKey key = keygen.generateKey();
            swt.util.HexString.printReadable(key.getEncoded());
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
*/

    public int balance() throws AuthFailedException {
        try {
            byte[] balance = {CLA_COPYCARD, INS_BALANCE,
                             (byte) 0x00, (byte) 0x00,
                             (byte) 0x02};

            byte[] response = sc.transmit(balance);
            if (response.length == 4 &&
                ByteArray.getShort(response, 2) == ISO7816.SW_OK) {
                return ByteArray.getShort(response, 0) & 0xFFFF;
            } else if (ByteArray.getShort(response, 0) == ERR_NO_AUTH) {
                auth();
                return balance();
            } else {
                System.out.println("Error in Engine.balance()!");
                System.out.println("CardResponse: " +
                                   swt.util.HexString.hexify(response));
            }
        } catch (swt.swtTerminalException ex) {
            ex.printStackTrace();
        }
        return -1;
    }


    public int load(int credits) throws Exception {
        try {
            byte lo = (byte) (credits & 0xFF);
            byte hi = (byte) (credits >> 8);

            cip.init(javax.crypto.Cipher.ENCRYPT_MODE, the_key);

            //die Zufallszahl und credits verschlsseln
            byte[] enc = cip.doFinal(new byte[] {nonce[0], nonce[1], nonce[2],
                                     nonce[3], nonce[4], nonce[5], hi, lo});

            byte[] load = {CLA_COPYCARD, INS_LOAD,
                          (byte) 0x00, (byte) 0x00,
                          (byte) 0x08,
                          enc[0], enc[1], enc[2], enc[3],
                          enc[4], enc[5], enc[6], enc[7],
                          (byte) 0x02};

            byte[] response = sc.transmit(load);

            if (response.length == 4 &&
                ByteArray.getShort(response, 2) == ISO7816.SW_OK) {
                return ByteArray.getShort(response, 0) & 0xFFFF;
            } else if (ByteArray.getShort(response, 0) == ERR_NO_AUTH) {
                auth();
                return load(credits);
            } else if (ByteArray.getShort(response, 0) == ERR_WRONG_NONCE) {
                throw new WrongNonceException();
            } else if (ByteArray.getShort(response, 0) == ERR_OVER_LIMIT) {
                throw new OverLimitException();
            } else {
                System.out.println("Error in Engine.load()!");
                System.out.println("CardResponse: " +
                                   swt.util.HexString.hexify(response));
            }
        } catch (swt.swtTerminalException ex) {
            ex.printStackTrace();
        }
        return -1;
    }


    public int pay (int credits) throws Exception {

        try {
            byte lo = (byte) (credits & 0xFF);
            byte hi = (byte) (credits >> 8);

            cip.init(javax.crypto.Cipher.ENCRYPT_MODE, the_key);

            //die Zufallszahl und credits verschlsseln
            byte[] enc = cip.doFinal(new byte[] {nonce[0], nonce[1], nonce[2],
                         nonce[3], nonce[4], nonce[5], hi, lo});

            byte[] pay = {CLA_COPYCARD, INS_PAY,
                         (byte) 0x00, (byte) 0x00,
                         (byte) 0x08,
                         enc[0], enc[1], enc[2], enc[3],
                         enc[4], enc[5], enc[6], enc[7],
                         (byte) 0x02};

            byte[] response = sc.transmit(pay);

            if (response.length == 4 &&
                ByteArray.getShort(response, 2) == ISO7816.SW_OK) {
                return ByteArray.getShort(response, 0) & 0xFFFF;
            } else if (ByteArray.getShort(response, 0) == ERR_NO_AUTH) {
                auth();
                return pay(credits);
            } else if (ByteArray.getShort(response, 0) == ERR_WRONG_NONCE) {
                throw new WrongNonceException();
            } else if (ByteArray.getShort(response, 0) == ERR_LOW_CREDIT) {
                throw new LowCreditException();
            } else {
                System.out.println("Error in Engine.pay()!");
                System.out.println("CardResponse: " +
                                   swt.util.HexString.hexify(response));
            }
        } catch (swt.swtTerminalException ex) {
            ex.printStackTrace();
        }
        return -1;
    }


    public void auth() throws AuthFailedException {
        try {
            SecureRandom rand = new SecureRandom();
            rand.nextBytes(nonce);

            // ruft Methode authTerm auf der Karte auf und bergibt die unverschlsselte Zufallszahl
            byte[] auth_card = {CLA_COPYCARD, INS_AUTH_CARD,
                               (byte) 0x00, (byte) 0x00,
                               (byte) 0x06,
                               nonce[0], nonce[1], nonce[2],
                               nonce[3], nonce[4], nonce[5],
                               (byte) 0x0C};

            // Karte schickt Zufallszahl verschlsselt zurck und ihre selbst erzeugte unverschlsselte Zufallszahl mit
            byte[] response = sc.transmit(auth_card);

            if (response.length < 14) {
                System.out.println("Error in Engine.auth()!");
                System.out.println("auth_card_wrong_answer");
                throw new AuthFailedException();
            }

            cip.init(javax.crypto.Cipher.DECRYPT_MODE, the_key);

            //von der Karte verschlsselte Zufallszahl entschlsseln
            byte[] dec = cip.doFinal(response, 0, 8);
            if (!ByteArray.equals(nonce, 0, dec, 0, 6)) {
                throw new CardAuthFailedException();
            }

            cip.init(javax.crypto.Cipher.ENCRYPT_MODE, the_key);

            //die Zufallszahl der Karte verschlsseln und zurcksenden
            byte[] enc = cip.doFinal(new byte[] {dec[6], dec[7], response[8],
                                     response[9], response[10], response[11], 0,
                                     0});
            byte[] auth_term = {CLA_COPYCARD, INS_AUTH_TERM,
                               (byte) 0x00, (byte) 0x00,
                               (byte) 0x08,
                               enc[0], enc[1], enc[2], enc[3],
                               enc[4], enc[5], enc[6], enc[7]};

            response = sc.transmit(auth_term);

            if (ByteArray.getShort(response, 0) != ISO7816.SW_OK) {
                throw new TermAuthFailedException();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new AuthFailedException();
        }

/*        } catch (InvalidKeyException e) {
            System.out.println(e);
        } catch (javax.crypto.IllegalBlockSizeException e) {
            System.out.println(e);
        } catch (javax.crypto.BadPaddingException e) {
            System.out.println(e);
        } catch (swt.swtTerminalException e) {
            System.out.println(e);
        }
*/
    }
}


class AuthFailedException extends Exception {}
class CardAuthFailedException extends AuthFailedException {}
class TermAuthFailedException extends AuthFailedException {}

class OverLimitException extends Exception {}
class LowCreditException extends Exception {}
class WrongNonceException extends Exception {}
